@@ -30,6 +30,11 @@ def dispatch(host, port, protocol, database_name, user, session, object_type,
 
     if object_type == 'common':
         if method == 'login':
+            try:
+                database = Database(database_name).connect()
+                cursor = database.cursor()
+            except Exception:
+                return False
             res = security.login(database_name, user, session)
             Cache.clean(database_name)
             logger = logging.getLogger('dispatcher')
@@ -51,12 +56,14 @@ def dispatch(host, port, protocol, database_name, user, session, object_type,
             return time.tzname[0]
         elif method == 'list_lang':
             return [
+                ('bg_BG', 'Български'),
                 ('cs_CZ', 'Čeština'),
                 ('de_DE', 'Deutsch'),
                 ('en_US', 'English'),
                 ('es_ES', 'Español (España)'),
                 ('es_CO', 'Español (Colombia)'),
                 ('fr_FR', 'Français'),
+                ('nl_NL', 'Nederlands'),
                 ('ru_RU', 'Russian'),
             ]
         elif method == 'db_exist':
@@ -68,6 +75,8 @@ def dispatch(host, port, protocol, database_name, user, session, object_type,
             except Exception:
                 return False
         elif method == 'list':
+            if CONFIG['prevent_dblist']:
+                raise Exception('AccessDenied')
             database = Database().connect()
             try:
                 cursor = database.cursor()
